@@ -19,6 +19,7 @@ local core = require("apisix.core")
 local get_uri_args = ngx.req.get_uri_args
 local route = require("apisix.utils.router")
 local plugin = require("apisix.plugin")
+local standalone = require("apisix.admin.standalone")
 local v3_adapter = require("apisix.admin.v3_adapter")
 local utils = require("apisix.admin.utils")
 local ngx = ngx
@@ -249,10 +250,8 @@ local function run()
         else
             core.response.set_header("X-API-VERSION", "v2")
         end
-        if resource.need_v3_filter then
-            data = v3_adapter.filter(data)
-        end
 
+        data = v3_adapter.filter(data, resource)
         data = strip_etcd_resp(data)
 
         core.response.exit(code, data)
@@ -418,12 +417,21 @@ local function schema_validate()
 end
 
 
+local function standalone_run()
+    set_ctx_and_check_token()
+    return standalone.run()
+end
+
+
+local http_head_route = {
+    paths = [[/apisix/admin]],
+    methods = {"HEAD"},
+    handler = head,
+}
+
+
 local uri_route = {
-    {
-        paths = [[/apisix/admin]],
-        methods = {"HEAD"},
-        handler = head,
-    },
+    http_head_route,
     {
         paths = [[/apisix/admin/*]],
         methods = {"GET", "PUT", "POST", "DELETE", "PATCH"},
@@ -453,13 +461,30 @@ local uri_route = {
 }
 
 
+local standalone_uri_route = {
+    http_head_route,
+    {
+        paths = [[/apisix/admin/configs]],
+        methods = {"GET", "PUT"},
+        handler = standalone_run,
+    },
+}
+
+
 function _M.init_worker()
     local local_conf = core.config.local_conf()
     if not local_conf.apisix or not local_conf.apisix.enable_admin then
         return
     end
 
-    router = route.new(uri_route)
+    local is_yaml_config_provider = local_conf.deployment.config_provider == "yaml"
+
+    if is_yaml_config_provider then
+        router = route.new(standalone_uri_route)
+        standalone.init_worker()
+    else
+        router = route.new(uri_route)
+    end
 
     -- register reload plugin handler
     events = require("apisix.events")
@@ -473,6 +498,10 @@ function _M.init_worker()
                 "please enable `admin_key_required` and set a secure admin key!")
         end
 
+        if is_yaml_config_provider then -- standalone mode does not need sync to etcd
+            return
+        end
+
         local ok, err = ngx_timer_at(0, function(premature)
             if premature then
                 return
