@@ -73,5 +73,9 @@ return resource.new({
     kind = "stream route",
     schema = core.schema.stream_route,
     checker = check_conf,
-    unsupported_methods = {"patch"}
+    unsupported_methods = { "patch" },
+    list_filter_fields = {
+        service_id = true,
+        upstream_id = true,
+    },
 })
