@@ -24,7 +24,16 @@ end
 local config_provider = local_conf.deployment and local_conf.deployment.config_provider
                       or "etcd"
 log.info("use config_provider: ", config_provider)
-local config = require("apisix.core.config_" .. config_provider)
+
+local config
+-- Currently, we handle JSON parsing in config_yaml, so special processing is needed here.
+if config_provider == "json" then
+    config = require("apisix.core.config_yaml")
+    config.file_type = "json"
+else
+    config = require("apisix.core.config_" .. config_provider)
+end
+
 config.type = config_provider
 
 
