@@ -58,6 +58,7 @@ local str_sub         = string.sub
 local tonumber        = tonumber
 local type            = type
 local pairs           = pairs
+local tostring       = tostring
 local ngx_re_match    = ngx.re.match
 local control_api_router
 
@@ -165,7 +166,7 @@ function _M.http_exit_worker()
 end
 
 
-function _M.http_ssl_phase()
+function _M.ssl_phase()
     local ok, err = router.router_ssl.set(ngx.ctx.matched_ssl)
     if not ok then
         if err then
@@ -176,7 +177,7 @@ function _M.http_ssl_phase()
 end
 
 
-function _M.http_ssl_client_hello_phase()
+function _M.ssl_client_hello_phase()
     local sni, err = apisix_ssl.server_name(true)
     if not sni or type(sni) ~= "string" then
         local advise = "please check if the client requests via IP or uses an outdated " ..
@@ -211,6 +212,10 @@ function _M.http_ssl_client_hello_phase()
         core.log.error("failed to set ssl protocols: ", err)
         ngx_exit(-1)
     end
+
+    -- in stream subsystem, ngx.ssl.server_name() return hostname of ssl session in preread phase,
+    -- so that we can't get real SNI without recording it in ngx.ctx during client_hello phase
+    ngx.ctx.client_hello_sni = sni
 end
 
 
@@ -295,13 +300,25 @@ local function set_upstream_headers(api_ctx, picked_server)
 end
 
 
-local function verify_tls_client(ctx)
-    if apisix_base_flags.client_cert_verified_in_handshake then
-        -- For apisix-runtime, there is no need to rematch SSL rules as the invalid
-        -- connections are already rejected in the handshake
-        return true
+-- verify the TLS session resumption by checking if the SNI in the client hello
+-- matches the hostname of the SSL session, this is to prevent the mTLS bypass security issue.
+local function verify_tls_session_resumption()
+    local session_hostname, err = apisix_ssl.session_hostname()
+    if err then
+        core.log.error("failed to get session hostname: ", err)
+        return false
+    end
+    if session_hostname and session_hostname ~= ngx.ctx.client_hello_sni then
+        core.log.error("sni in client hello mismatch hostname of ssl session, ",
+                         "sni: ", ngx.ctx.client_hello_sni, ", hostname: ", session_hostname)
+        return false
     end
 
+    return true
+end
+
+
+local function verify_tls_client(ctx)
     local matched = router.router_ssl.match_and_set(ctx, true)
     if not matched then
         return true
@@ -319,6 +336,10 @@ local function verify_tls_client(ctx)
 
             return false
         end
+
+        if not verify_tls_session_resumption() then
+            return false
+        end
     end
 
     return true
@@ -390,6 +411,10 @@ local function verify_https_client(ctx)
                            ", but the host is ", host)
             return false
         end
+
+        if not verify_tls_session_resumption() then
+            return false
+        end
     end
 
     return true
@@ -822,9 +847,18 @@ local function healthcheck_passive(api_ctx)
     local host = up_conf.checks and up_conf.checks.active
                  and up_conf.checks.active.host
     local port = up_conf.checks and up_conf.checks.active
-                 and up_conf.checks.active.port
+                 and up_conf.checks.active.port or api_ctx.balancer_port
 
     local resp_status = ngx.status
+
+    if not is_http then
+        -- 200 is the only success status code for TCP
+        if resp_status ~= 200 then
+            checker:report_tcp_failure(api_ctx.balancer_ip, port, host, nil, "passive")
+        end
+        return
+    end
+
     local http_statuses = passive and passive.healthy and
                           passive.healthy.http_statuses
     core.log.info("passive.healthy.http_statuses: ",
@@ -833,7 +867,7 @@ local function healthcheck_passive(api_ctx)
         for i, status in ipairs(http_statuses) do
             if resp_status == status then
                 checker:report_http_status(api_ctx.balancer_ip,
-                                           port or api_ctx.balancer_port,
+                                           port,
                                            host,
                                            resp_status)
             end
@@ -851,7 +885,7 @@ local function healthcheck_passive(api_ctx)
     for i, status in ipairs(http_statuses) do
         if resp_status == status then
             checker:report_http_status(api_ctx.balancer_ip,
-                                       port or api_ctx.balancer_port,
+                                       port,
                                        host,
                                        resp_status)
         end
@@ -859,6 +893,61 @@ local function healthcheck_passive(api_ctx)
 end
 
 
+function _M.status()
+    core.response.exit(200, core.json.encode({ status = "ok" }))
+end
+
+function _M.status_ready()
+    local local_conf = core.config.local_conf()
+    local role = core.table.try_read_attr(local_conf, "deployment", "role")
+    local provider = core.table.try_read_attr(local_conf, "deployment", "role_" ..
+                                              role, "config_provider")
+    if provider == "yaml" or provider == "etcd" then
+        local status_shdict = ngx.shared["status-report"]
+        local ids = status_shdict:get_keys()
+        local error
+        local worker_count = ngx.worker.count()
+       if #ids ~= worker_count then
+            core.log.warn("worker count: ", worker_count, " but status report count: ", #ids)
+            error = "worker count: " .. ngx.worker.count() ..
+            " but status report count: " .. #ids
+        end
+        if error then
+            core.response.exit(503, core.json.encode({
+                status = "error",
+                error = error
+            }))
+            return
+        end
+        for _, id in ipairs(ids) do
+            local ready = status_shdict:get(id)
+            if not ready then
+                core.log.warn("worker id: ", id, " has not received configuration")
+                error = "worker id: " .. id ..
+                                  " has not received configuration"
+                break
+            end
+        end
+
+        if error then
+            core.response.exit(503, core.json.encode({
+                status = "error",
+                error = error
+            }))
+            return
+        end
+
+        core.response.exit(200, core.json.encode({ status = "ok" }))
+        return
+    end
+
+    core.response.exit(503, core.json.encode({
+        status = "error",
+        message = "unknown config provider: " .. tostring(provider)
+    }), { ["Content-Type"] = "application/json" })
+end
+
+
 function _M.http_log_phase()
     local api_ctx = common_phase("log")
     if not api_ctx then
@@ -957,25 +1046,6 @@ function _M.http_control()
 end
 
 
-function _M.stream_ssl_phase()
-    local ngx_ctx = ngx.ctx
-    local api_ctx = core.tablepool.fetch("api_ctx", 0, 32)
-    ngx_ctx.api_ctx = api_ctx
-
-    local ok, err = router.router_ssl.match_and_set(api_ctx)
-
-    core.tablepool.release("api_ctx", api_ctx)
-    ngx_ctx.api_ctx = nil
-
-    if not ok then
-        if err then
-            core.log.error("failed to fetch ssl config: ", err)
-        end
-        ngx_exit(-1)
-    end
-end
-
-
 function _M.stream_init(args)
     core.log.info("enter stream_init")
 
@@ -1169,6 +1239,8 @@ function _M.stream_log_phase()
         return
     end
 
+    healthcheck_passive(api_ctx)
+
     core.ctx.release_vars(api_ctx)
     if api_ctx.plugins then
         core.tablepool.release("plugins", api_ctx.plugins)
