@@ -494,22 +494,35 @@ function _M.filter(ctx, conf, plugins, route_conf, phase)
             goto continue
         end
 
-        if not check_disable(plugin_conf) then
-            if plugin_obj.run_policy == "prefer_route" and route_plugin_conf ~= nil then
-                local plugin_conf_in_route = route_plugin_conf[name]
-                local disable_in_route = check_disable(plugin_conf_in_route)
-                if plugin_conf_in_route and not disable_in_route then
-                    goto continue
-                end
-            end
-
-            if plugin_conf._meta and plugin_conf._meta.priority then
-                custom_sort = true
-            end
-            core.table.insert(plugins, plugin_obj)
-            core.table.insert(plugins, plugin_conf)
+        if check_disable(plugin_conf) then
+            goto continue
         end
 
+        if plugin_obj.run_policy == "prefer_route" and route_plugin_conf ~= nil then
+            local plugin_conf_in_route = route_plugin_conf[name]
+            local disable_in_route = check_disable(plugin_conf_in_route)
+            if plugin_conf_in_route and not disable_in_route then
+                goto continue
+            end
+        end
+
+        -- in the rewrite phase, the plugin executes in the following order:
+        -- 1. execute the rewrite phase of the plugins on route(including the auth plugins)
+        -- 2. merge plugins from consumer and route
+        -- 3. execute the rewrite phase of the plugins on consumer(phase: rewrite_in_consumer)
+        -- in this case, we need to skip the plugins that was already executed(step 1)
+        if phase == "rewrite_in_consumer"
+                and (not plugin_conf._from_consumer or plugin_obj.type == "auth") then
+            plugin_conf._skip_rewrite_in_consumer = true
+        end
+
+        if plugin_conf._meta and plugin_conf._meta.priority then
+            custom_sort = true
+        end
+
+        core.table.insert(plugins, plugin_obj)
+        core.table.insert(plugins, plugin_conf)
+
         ::continue::
     end
 
@@ -523,15 +536,6 @@ function _M.filter(ctx, conf, plugins, route_conf, phase)
             local plugin_obj = plugins[i]
             local plugin_conf = plugins[i + 1]
 
-            -- in the rewrite phase, the plugin executes in the following order:
-            -- 1. execute the rewrite phase of the plugins on route(including the auth plugins)
-            -- 2. merge plugins from consumer and route
-            -- 3. execute the rewrite phase of the plugins on consumer(phase: rewrite_in_consumer)
-            -- in this case, we need to skip the plugins that was already executed(step 1)
-            if phase == "rewrite_in_consumer" and not plugin_conf._from_consumer then
-                plugin_conf._skip_rewrite_in_consumer = true
-            end
-
             tmp_plugin_objs[plugin_conf] = plugin_obj
             core.table.insert(tmp_plugin_confs, plugin_conf)
 
@@ -1179,20 +1183,13 @@ function _M.run_plugin(phase, plugins, api_ctx)
         and phase ~= "delayed_body_filter"
     then
         for i = 1, #plugins, 2 do
-            local phase_func
-            if phase == "rewrite_in_consumer" then
-                if plugins[i].type == "auth" then
-                    plugins[i + 1]._skip_rewrite_in_consumer = true
-                end
-                phase_func = plugins[i]["rewrite"]
-            else
-                phase_func = plugins[i][phase]
-            end
 
             if phase == "rewrite_in_consumer" and plugins[i + 1]._skip_rewrite_in_consumer then
                 goto CONTINUE
             end
 
+            local phase_func = phase == "rewrite_in_consumer" and plugins[i]["rewrite"]
+                               or plugins[i][phase]
             if phase_func then
                 local conf = plugins[i + 1]
                 if not meta_filter(api_ctx, plugins[i]["name"], conf)then
