@@ -17,7 +17,7 @@
 local core     = require("apisix.core")
 local consumer_mod = require("apisix.consumer")
 local plugin_name = "key-auth"
-
+local schema_def = require("apisix.schema_def")
 
 local schema = {
     type = "object",
@@ -33,7 +33,8 @@ local schema = {
         hide_credentials = {
             type = "boolean",
             default = false,
-        }
+        },
+        anonymous_consumer = schema_def.anonymous_consumer_schema,
     },
 }
 
@@ -65,8 +66,7 @@ function _M.check_schema(conf, schema_type)
     end
 end
 
-
-function _M.rewrite(conf, ctx)
+local function find_consumer(ctx, conf)
     local from_header = true
     local key = core.request.header(ctx, conf.header)
 
@@ -77,18 +77,13 @@ function _M.rewrite(conf, ctx)
     end
 
     if not key then
-        return 401, {message = "Missing API key in request"}
+        return nil, nil, "Missing API key in request"
     end
 
-    local consumer_conf = consumer_mod.plugin(plugin_name)
-    if not consumer_conf then
-        return 401, {message = "Missing related consumer"}
-    end
-
-    local consumers = consumer_mod.consumers_kv(plugin_name, consumer_conf, "key")
-    local consumer = consumers[key]
+    local consumer, consumer_conf, err = consumer_mod.find_consumer(plugin_name, "key", key)
     if not consumer then
-        return 401, {message = "Invalid API key in request"}
+        core.log.warn("failed to find consumer: ", err or "invalid api key")
+        return nil, nil, "Invalid API key in request"
     end
     core.log.info("consumer: ", core.json.delay_encode(consumer))
 
@@ -102,6 +97,25 @@ function _M.rewrite(conf, ctx)
         end
     end
 
+    return consumer, consumer_conf
+end
+
+
+function _M.rewrite(conf, ctx)
+    local consumer, consumer_conf, err = find_consumer(ctx, conf)
+    if not consumer then
+        if not conf.anonymous_consumer then
+            return 401, { message = err}
+        end
+        consumer, consumer_conf, err = consumer_mod.get_anonymous_consumer(conf.anonymous_consumer)
+        if not consumer then
+            err = "key-auth failed to authenticate the request, code: 401. error: " .. err
+            core.log.error(err)
+            return 401, { message = "Invalid user authorization"}
+        end
+    end
+
+    core.log.info("consumer: ", core.json.delay_encode(consumer))
     consumer_mod.attach_consumer(ctx, consumer, consumer_conf)
     core.log.info("hit key-auth rewrite")
 end
