@@ -55,14 +55,12 @@ local schema = {
         },
         http_to_https = {type = "boolean"},
         encode_uri = {type = "boolean", default = false},
-        append_query_string = {type = "boolean"},
+        append_query_string = {type = "boolean", default = false},
     },
     oneOf = {
         {required = {"uri"}},
         {required = {"regex_uri"}},
-        {required = {"http_to_https"}, ["not"] = {
-           required = {"append_query_string"}
-        }}
+        {required = {"http_to_https"}}
     }
 }
 
@@ -103,6 +101,7 @@ end
 
 function _M.check_schema(conf)
     local ok, err = core.schema.check(schema, conf)
+
     if not ok then
         return false, err
     end
@@ -117,6 +116,10 @@ function _M.check_schema(conf)
         end
     end
 
+    if conf.http_to_https and conf.append_query_string then
+        return false, "only one of `http_to_https` and `append_query_string` can be configured."
+    end
+
     return true
 end
 
@@ -163,11 +166,6 @@ local function get_port(attr)
         return port
     end
 
-    port = ssl["listen_port"]
-    if port then
-        return port
-    end
-
     local ports = ssl["listen"]
     if ports and #ports > 0 then
         local idx = math_random(1, #ports)
